version 12.1
clear all
set more off

* Define path to files used during the exercise sessions

global path_d="T:\Labor_economics\STATA_12_manuals\STATA_12_book\Datasets"
global path_w="T:\Labor_economics\ES\Datasets_working"

* ### 2. Data Management - Part I ###
* ###################################

* #################################
* ### 2.1 Creating New Datasets ###

* ### 2.1.1 Creating a New Dataset by Typing in Data ###

* To create a dataset from printed information like Table 2.1 is through the 
* # Data Editor, invoked by clicking respectide Button
* # selecting Window > Data Editor from the menu bar
* # Typing the command "edit"

// We can assign more descriptive variable names by double-clicking on the column headings (such as var1) and then typing a new name in the resulting dialog box; 
// Eight characters or fewer works best,  names with up to 32 characters are allowed. 
// We can also create variable labels that contain a brief description. 
// For example, var2 (population) might be renamed pop, and given the variable label Population in 1000s, 1995.

// Renaming and labeling variables can also be done outside of the Data Editor through the rename and label variable commands:
* # "rename var1 place"
* # "rename var2 pop"
* # "rename var3 unemp"
* # "rename var4 mlife"
* # "rename var5 flife"

* Alternatively, the five rename operations could be accomplished in one step:
// "rename (var1 var2 var3 var4 var5) (place pop unemp mlife flife)"

* # "label data "Canadian dataset 1""
* # "label variable place "Place name"
* # "label variable pop "Population in 1000s, 1995""
* # "label variable unemp "% 15+ population unemployed, 1995""
* # "label variable mlife "Male life expectancy years""
* # "label variable flife "Female life expectancy years""

// Numbers should not include any commas!
// String variable values can be almost any combination of letters, numbers, symbols or spaces up to 244 characters long.


* ### 2.1.2 Creating a New Dataset by Copy and Paste ###

* Given Canada1.xlsx


* ##############################################
* ### 2.1.3 Importing Data from Other Programs ###

* # import excel

* Stata can import data from Excel spreadsheets directly through menu selections
* # File > Import > Excel spreadsheet (*.xls; *.xlsx)
* # or the "import excel" command. 

// Spreadsheets often contain titles, notes, subtables, multiple sheets, graphs or other features
// To restrict the import operation to a particular range of cells, use a "cellrange()" option. 
// The "sheet()" option can specify what sheet within the spreadsheet to import. 
// A "firstrow" option tells Stata that the first row of these cells contain variables names. 

* Given snowfall.xls
import excel using $path_d\snowfall.xls, sheet("Berlin") cellrange(a4:o56) firstrow clear
describe

* To meet Stata criteria 
* # Start variable names with a letter or underscore, and have no embedded blanks. 
* # Missing values should be replaced with blanks or numerical codes, 
// and non-numeric characters removed from cells in columns meant to represent numerical variables.

* # insheet

* Text file MEI.raw contains monthly values of MEI, Multivariate ENSO Index (MEI) describing tropical Pacific conditions, from January 1950 through December 2011.
// These are tab-separated values
// The first row of the text file contains a list of variable names: mei1 for January MEI, mei2 for February, and so forth 
// The first few rows are shown in your handouts

// We can read these data into Stata using the insheet command, with options to specify that values are tab-separated, 
// and the first row contains variable names.

insheet using $path_d\MEI.raw, tab name clear
save $path_w\MEI0.dta, replace
describe

// With a comma option instead of tab, insheet could read a text file of comma-separated values,
// Text files can be read through Stata menus as well. Explore Data > Import to see the options available.

* # infix

// A different arrangement called fixed-column format has values that are not necessarily delimited at all, 
// but do occupy predefined column positions. The "infix" command can read such files. 
// We have to specify exactly how the columns should be read.

// Consider a text (ASCII) file named nfresour.raw in your handouts

// These data concern natural resource production in Newfoundland. 
// The four variables occupy fixed column positions: 
// * columns 14 are the years (1986...1991); 
// * columns 58 measure forestry production in thousands of cubic meters (2408...missing); 
// * columns 914 measure mine production in thousands of dollars (764,169...793,000); 
// * and columns 1518 are the consumer price index relative to 1986 (1000...1262). 

// Notice that in fixed-column format blanks indicate missing values. To read nfresour.raw into Stata:

infix year 1-4 wood 5-8 mines 9-14 CPI 15-18 using $path_d\nfresour.txt, clear
list

// Type "help import" to learn more about different methods:
// import excel, insheet, odbc, infile (free format), infix (fixed format), infile (fixed format), import sasxport, haver, xmluse

* To export data from Stata to other programs: 
// * File > Export > Excel spreadsheet (*.xls; *.xlsx)
// * Copy your data from Statas Data Editor or Data Browser and paste this directly into a spreadsheet such as Excel.
* The outsheet and outfile commands (or Data > Export) can write text files in several different formats. 


* #############################################
* ### 2.2 Combining Two or More Stata Files ###

* Combine Stata datasets in two general ways: 
* # "append" a second dataset that contains additional observations
* # "merge" with other datasets that contain new variables or values. 

* Append example

* File lakewin1.dta contains the ice-out dates for New Hampshires largest lake recorded from 1887 through 2007.

use $path_d\lakewin1.dta, clear
describe
list in -4/l

* File lakewin2.dta contains newer data from 2008 through 2012. 
// It has the same variables and format, so we can combine the update in lakewin2.dta with the older information in lakewin1.dta, using the "append" command.

use $path_d\lakewin2.dta, clear
describe
list

append using $path_d\lakewin1.dta

sort year
label data "Lake Winnipesaukee ice out 1887-2012"
save $path_w\lakewin3, replace
list in -7/l

* Merge example

* File lakesun.dta contains ice out dates for New Hampshires lake over the years 1869 through 2012. 
// Lake Sunapee (lakesun.dta) and Lake Winnipesaukee (lakewin3.dta) both form yearly series that could easily be combined into one dataset. 
// We do this with the "merge 1:1 year" command.

use $path_d\lakesun.dta, clear
sort year
describe
merge 1:1 year using $path_d\lakewin3.dta

label data "Sunapee & Winnipesaukee ice out 1869-2012"
describe
save $path_w\lakesunwin.dta, replace

// The merge results tell us that 126 years were present in both the master dataset (the data currently in memory  lakesun.dta in this example) 
// and the using dataset (lakewin3.dta).
// A further 18 years (1869 to 1886) existed only in lakesun.dta, so the Lake Winnipesaukee variables will have missing values in those years. 

* "merge" commands create a variable named _merge that records whether the observation came from:
* # the master data only (_merge = 1)
* # the using data only (_merge = 2)
* # both (_merge = 3)

// By default, whenever the same variables are found in both datasets, those of the master data are retained and those of the using data ignored. 

* "merge" options:
* # "merge 1:1 year using newdata.dta, update"
// A command would allow any missing values in the master data to be replaced by corresponding nonmissing values found in the using data (here, newdata.dta)

* # "merge 1:1 year using newdata, update replace"
// A command causes any values from the master data to be replaced by nonmissing values from the using data, if the latter are different

// Also possible are 1-to-many (1:m), many-to-1 (m:1) or many-to-many (m:m) operations. 
// Type "help merge" for details
// Merging and appending data can be accomplished through Data > Combine datasets menus as well.

*** "append" might be compared to lengthening a sheet of paper (that is, the dataset in memory) 
// by taping a second sheet with new observations (rows) to its bottom. 
*** "merge", in its simplest form, corresponds to widening our sheet of paper 
// by taping a second sheet to its right side, thereby adding new variables (columns).


version 12.1
clear all
set more off
// cd "T:\Labor_economics\ES\ES2\Datasets"

cd "D:\Study\CERGE-EI\TA\Labor_economics\ES\ES2\Datasets"

* ###################################################
* ### 2.3 Collapsing Data [important extra topic] ###

* "collapse" aggregates data into means, medians or other statistics for groups defined by one or more variables.

* Consider data on monthly global temperatures from January 1880 to December 2011 (global2.dta)

use global2.dta, clear
describe

// With collapse, we could build a simplified dataset containing mean temperature anomalies for 132 years instead of 1,584 separate months.

collapse (mean) temp, by(year)
label variable temp "NCDC annual mean temp anomaly, deg C"
save global_yearly.dta, replace
describe

* "collapse" can create variables based on any of the following summary statistics:
* # "mean" Means (default, if statistic is not specified)
* # "median" Medians
* # "p1" 1st percentiles
* # "p2" 2nd percentiles (and so forth to p99)
* # "sd" Standard deviations
* # "semean" Standard error of the mean (sd/sqrt(n))
* # "sebinomial" Standard error of the mean, binomial (sqrt(p(1p)/n))
* # "sepoisson" Standard error of the mean, Poisson (sqrt(mean))
* # "sum" Sums
* # "rawsum" Sums, ignoring optionally specified weight
* # "count" Number of nonmissing observations
* # "max" Maximums
* # "min" Minimums
* # "iqr" Interquartile range
* # "first" First values
* # "last" Last values
* # "firstnm" First nonmissing values
* # "lastnm" Last nonmissing values


* ########################################################################################
* ### 2.4 Specifying Subsets of the Data: in and if Qualifiers [important extra topic] ###

* Given global1.dta
// National Climate Data Center (NCDC) produces estimates of global temperature anomalies (deviations from the 19012000 mean, in degrees Celsius) 
// for every month back to January 1880.

use global1.dta, clear

* To list the 1st through 5th observations, type
list in 1/5

* Which 10 months had the highest temperature anomalies?
 // To find out, we first sort from lowest to highest by temperature, then list the 10th-from-last to last observations:
sort temp
list in -10/l
// The letter l denotes the last case, and 10 , for example, the tenth-from-last.

* To summarize January and February temperature anomalies for the years from 1940 through 1969
summarize temp if (month == 1 | month == 2) & year >= 1940 & year < 1970
// Parentheses allow us to specify the precedence among multiple operators.

* relational operators:
// == is equal to
// != is not equal to (~= also works)
// > is greater than
// < is less than
// >= is greater than or equal to
// <= is less than or equal to

* logical operators:
// & and
// | or (symbol is a vertical bar, not the number one or letter el)
// ! not (~ also works)

* If and missing values

// In some operations (notably sort and if, although not in statistical calculations such as means or correlations), 
// the missing values are treated as if they were large positive numbers.
// For example, suppose that we are analyzing opinion poll data. A command such as the following would tabulate 
// vote not only for people age 65 and older, as intended, but also for any people whose age values are missing:

// "tabulate vote if age >= 65"

// Where missing values exist, we often need to deal with them explicitly in the if expression.

// "tabulate vote if age >= 65 & !missing(age)"

// The not missing() function "!missing( )" provides a general way to select observations with nonmissing values. 
// There are several alternative ways to screen out missing values. 
// The missing( ) function evaluates to 1 if a value is missing, and 0 if it is not. 
// For example, to tabulate vote only for those observations that have nonmissing values of age, income and education, type

// "tabulate vote if missing(age, income, education)==0"

// Because the default missing value . is represented internally by a very large number, 
// and other missing values (described later) are even larger, a less than inequality <. can be used to screen all of them out:

// "tabulate vote if age <. & income <. & education <."


* ######################################################################
* ### 2.5 Generating and Replacing Variables [important extra topic] ###

* In Canada, as in most industrial societies, women tend to live longer than men.
// To analyze regional variations in this gender gap, we might retrieve dataset Canada1.dta 
// and generate a new variable equal to female life expectancy (flife) minus male life expectancy (mlife).

use Canada1.dta, clear
generate gap = flife - mlife
label variable gap "Female-male life expectancy gap"
describe gap

* To convert dollars to thousands of dollars, we divide all values by 1,000:
// "replace income = income/1000"

* Suppose our survey variables include age and year born. 
// "replace age = 2012-born if missing(age) | age+1 < 2012-born"
// This replaces values of variable age with 2012 minus the year of birth (born) if age is missing
// or if the reported age (plus one year) is less than 2012 minus the year of birth.

* generate and replace provide tools to create categorical variables as well. 
// The Canadian dataset includes several types of observations: 2 territories, 10 provinces and one country combining them all. 
// Create a categorical variable that indicates the observations type. 
// We start by generating type as a constant, equal to 1 for each observation. 
// Next, we replace this with the value 2 for the Yukon and Northwest Territories, and with 3 for Canada. 
// The final steps involve labeling new variable type and defining labels for values 1, 2 and 3.

use Canada1.dta, clear
generate type = 1
replace type = 2 if place == "Yukon" | place == "Northwest Territories"
replace type = 3 if place == "Canada"
label variable type "Province, territory or nation"
label define typelbl 1 "Province" 2 "Territory" 3 "Nation"
label values type typelbl
list

// The "label define" command specifies what labels go with what numbers. 
// The "label values" command specifies to which variable these labels apply. 

save Canada1.dta, replace


* ###############################
* ### 2.6 Missing Value Codes ###

* Dataset Granite2011_6.dta, contains data from a political opinion survey. 
// A question asking respondents about their level of interest regarding the 2012 general election 
// (genint) serves to illustrate Statas extended missing-value codes.

use Granite2011_6.dta, clear

tabulate genint

// We can see numerical values if we ask for the same table without value labels.

tabulate genint, nolabel

// A table of genint means by respondent education will be meaningless, because those 98 and 99 values have been averaged in.

tabulate educ, summarize(genint)

// Create improved version of variable genint, to be called genint2. 
// First, we reverse the 1 through 4 values so that higher values indicate greater interest instead of less interest.

generate genint2 = 5 - genint if genint <90

// Second, the 98 and 99 values should be identified as missing so they do not enter calculations for the mean and other statistics. 
// Here we use the missing value code .a to represent dont know responses, formerly coded 98. We use .b to represent no answer responses, formerly coded 99.

replace genint2 = .a if genint == 98
replace genint2 = .b if genint == 99

// Third, the value labels can be shortened from long phrases like extremely interested to something that will take up less space in graphs and tables.

label variable genint2 "Interest in 2012 election (new)"
label define genint2 1 "Not very" 2 "Somewhat" 3 "Very" 4 "Extremely" .a "DK" .b "NA"
label values genint2 genint2

// Tabulating old against new variables to be sure that our commands worked as intended.

tabulate genint genint2, miss /* "miss" treat missing values like other values */

// Now it is easy to see that the average level of interest in the election rises with education.

tabulate educ, summ(genint2)

// Codding the missing values could be done easily for a whole list of variables using an "mvdecode" command such as

mvdecode genint income age, mv(97=. \ 98=.a \ 99=.b)

// The example above would change any values of genint, income or age from 97 to ., from 98 to .a and so forth. 
// The .a and .b (through .z) missing values can accept value labels, but . by itself cannot.

// As usual, the changes we have made do not become permanent until our dataset is saved. 
// After so much recoding, it makes sense to save these data with a new name  
// in case, for some future reason, we want to take another look at the original raw data.

save Granite2011_6_mvdecode.dta, replace


* ###########################
* ### 2.7 Using functions ###

* See help for more details on the following functions
// "help math functions"
// "help density functions"
// "help date functions"
// "help matrix functions"
// "help random number functions"
// "help string functions"
// "help time series functions"
// "help programming functions"

* Multiple functions, operators and qualifiers can be combined in one command as needed.

// "display", "generate" and "replace" have direct access to Statas statistical results.
// Consider dataset Arctic9.dta. Variable "extent" represents the mean area covered by at least 15% sea ice in September each year 
// For 33 years of satellite observation, the overall September mean was about 6.52 million km2.

use $path_d\Arctic9.dta, clear
summarize extent

// After summarize, Stata temporarily stores this mean as a scalar named r(mean) .

display r(mean)

* Use this result to create variable extent0, defined as the anomaly or deviation from the 19792011 mean

gen extent0 = extent - r(mean)
summ extent extent0

* To see a complete list of the names and values currently saved

summarize extent
return list

* Command "egen" has its own set of functions to accomplish tasks not easily done by generate. 
// These include such things as creating new variables from the sums, maxima, minima, medians, 
// interquartile ranges, standardized values, ranks or moving averages of existing variables or expressions. 


* #########################################################
* ### 2.8 Converting Between Numeric and String Formats ###

* Dataset Canada2.dta contains one string variable, place. It also has a labeled categorical variable, type. Both seem to have nonnumeric values.

use $path_d\Canada2.dta, clear
list place type

// Beneath the labels, however, type remains a numeric variable
list place type, nolabel

* The "encode" command generates a labeled-numeric variable from a string variable. 
// The number 1 is given to the alphabetically first value of the string variable, 2 to the second, and so on. 

encode place, gen(placenum)

* The "decode" command generates a string variable using the values of a labeled numeric variable. 
// Create string variable typestr from numeric variable type:

decode type, gen(typestr)
list place placenum type typestr
list place placenum type typestr, nolabel

* To convert string values into their numeric counterparts, use the "real" function.
// Allows for identification of those values which cannot be represented by a number as missing values

* The "destring" command provides a more flexible method for converting string variables to numeric. 
// See "help destring" for information about syntax and options.


* ####################################################
* ### 2.9 Using Explicit Subscripts with Variables ###

use $path_d\global2.dta, clear

* _N represents the total number of observations. _n represents the observation number:
// _n = 1 for the first observation, _n = 2 for the second, and so on to the last observation ( _n = _N ). 
// Creates a new variable, caseID, which is equal to the number of each observation as presently sorted

generate caseID = _n

* Explicit subscripting and the _n system variable have particular relevance when our data form a series. 
// temp[ _n] denotes the value of the _nth observation. temp[ _n1] denotes the previous temperature, and temp[ _n+1] denotes the next. 
// Thus, we might define a new variable "diftemp", which is equal to the change in temp since the previous month:

generate diftemp = temp - temp[_n-1]
